@@ -1,3 +1,4 @@
+// clang-format off
 /*
  * KDiff3 - Text Diff And Merge Tool
  *
@@ -32,31 +33,34 @@ Optimizations: Skip unneeded steps.
 #include "SourceData.h"
 
 #include "CommentParser.h"
+#include "compat.h"
 #include "diff.h"
+#include "EncodedDataStream.h"
 #include "LineRef.h"
 #include "Logging.h"
 #include "Utils.h"
 
 #include <algorithm>         // for min
 #include <memory>
+#include <optional>
 #include <vector>            // for vector
 
 #include <QtGlobal>
 
+#include <QByteArray>
 #include <QProcess>
 #include <QScopedPointer>
 #include <QString>
 #include <QTemporaryFile>
 #include <QTextCodec>
-#include <QTextStream>
 #include <QVector>
 
-#include <KLocalizedString>
+extern std::unique_ptr<Options> gOptions;
 
 void SourceData::reset()
 {
     mFromClipBoard = false;
-    m_pEncoding = nullptr;
+    mEncoding = u8"UTF-8";
     m_fileAccess = FileAccess();
     m_normalData.reset();
     m_lmppData.reset();
@@ -77,8 +81,7 @@ void SourceData::setFilename(const QString& filename)
     }
     else
     {
-        FileAccess fa(filename);
-        setFileAccess(fa);
+        setFileAccess(FileAccess(filename));
     }
 }
 
@@ -97,11 +100,6 @@ bool SourceData::isValid() const
     return isEmpty() || hasData();
 }
 
-void SourceData::setOptions(const QSharedPointer<Options>& pOptions)
-{
-    m_pOptions = pOptions;
-}
-
 QString SourceData::getFilename() const
 {
     return m_fileAccess.absoluteFilePath();
@@ -131,12 +129,13 @@ void SourceData::setFileAccess(const FileAccess& fileAccess)
 
     mErrors.clear();
 }
-void SourceData::setEncoding(QTextCodec* pEncoding)
+
+void SourceData::setEncoding(const QByteArray& encoding)
 {
-    m_pEncoding = pEncoding;
+    mEncoding = encoding;
 }
 
-const QString SourceData::setData(const QString& data)
+void SourceData::setData(const QString& data)
 {
     mErrors.clear();
     // Create a temp file for preprocessing:
@@ -151,15 +150,13 @@ const QString SourceData::setData(const QString& data)
     if(!bSuccess)
     {
         mErrors.append(i18n("Writing clipboard data to temp file failed."));
-        return mErrors[0];
+        return;
     }
     else
     {
         m_aliasName = i18n("From Clipboard");
         mFromClipBoard = true;
     }
-
-    return u8"";
 }
 
 const std::shared_ptr<LineDataVector>& SourceData::getLineDataForDiff() const
@@ -179,9 +176,9 @@ const std::shared_ptr<LineDataVector>& SourceData::getLineDataForDisplay() const
     return m_normalData.m_v;
 }
 
-LineCount SourceData::getSizeLines() const
+LineType SourceData::getSizeLines() const
 {
-    return SafeInt<LineCount>(m_normalData.lineCount());
+    return SafeInt<LineType>(m_normalData.lineCount());
 }
 
 qint64 SourceData::getSizeBytes() const
@@ -249,6 +246,10 @@ bool SourceData::FileData::readFile(FileAccess& file)
         return true;
 
     mDataSize = file.sizeForReading();
+    /*
+        If the extra bytes are removed an unknown heap currption issue is triggered in the
+        diff code. I don't have time to track this down to its true root cause.
+    */
     m_pBuf = std::make_unique<char[]>(mDataSize + 100); // Alloc 100 byte extra: Safety hack, not nice but does no harm.
                                                         // Some extra bytes at the end of the buffer are needed by
                                                         // the diff algorithm. See also GnuDiff::diff_2_files().
@@ -278,21 +279,7 @@ bool SourceData::FileData::readFile(const QString& filename)
     }
 
     FileAccess fa(filename);
-
-    if(!fa.isNormal())
-        return true;
-
-    mDataSize = fa.sizeForReading();
-    m_pBuf = std::make_unique<char[]>(mDataSize + 100); // Alloc 100 byte extra: Safety hack, not nice but does no harm.
-                                                        // Some extra bytes at the end of the buffer are needed by
-                                                        // the diff algorithm. See also GnuDiff::diff_2_files().
-    bool bSuccess = fa.readFile(m_pBuf.get(), mDataSize);
-    if(!bSuccess)
-    {
-        m_pBuf = nullptr;
-        mDataSize = 0;
-    }
-    return bSuccess;
+    return readFile(fa);
 }
 
 bool SourceData::saveNormalDataAs(const QString& fileName)
@@ -312,7 +299,7 @@ bool SourceData::FileData::writeFile(const QString& filename)
     return bSuccess;
 }
 
-//Depriated
+//Deprecated
 void SourceData::FileData::copyBufFrom(const FileData& src) //TODO: Remove me.
 {
     reset();
@@ -322,7 +309,7 @@ void SourceData::FileData::copyBufFrom(const FileData& src) //TODO: Remove me.
     memcpy(m_pBuf.get(), src.m_pBuf.get(), mDataSize);
 }
 
-QTextCodec* SourceData::detectEncoding(const QString& fileName, QTextCodec* pFallbackCodec)
+std::optional<const QByteArray> SourceData::detectEncoding(const QString& fileName)
 {
     QFile f(fileName);
     if(f.open(QIODevice::ReadOnly))
@@ -331,29 +318,28 @@ QTextCodec* SourceData::detectEncoding(const QString& fileName, QTextCodec* pFal
 
         qint64 size = f.read(buf, sizeof(buf));
         FileOffset skipBytes = 0;
-        QTextCodec* pCodec = detectEncoding(buf, size, skipBytes);
-        if(pCodec)
-            return pCodec;
+        return detectEncoding(buf, size, skipBytes);
     }
-    return pFallbackCodec;
+    return {};
 }
 
-void SourceData::readAndPreprocess(QTextCodec* pEncoding, bool bAutoDetectUnicode)
+void SourceData::readAndPreprocess(const char* encoding, bool bAutoDetect)
 {
-    m_pEncoding = pEncoding;
     QTemporaryFile fileIn1, fileOut1;
     QString fileNameIn1;
     QString fileNameOut1;
     QString fileNameIn2;
     QString fileNameOut2;
 
+    mEncoding = encoding;
+
     // Detect the input for the preprocessing operations
     if(!mFromClipBoard)
     {
         //Routine result of directory compare finding a file that isn't in all locations.
         if(!m_fileAccess.isValid()) return;
 
-        assert(!m_fileAccess.isDir());
+        assert(!m_fileAccess.exists() || !m_fileAccess.isDir());
         if(!m_fileAccess.isNormal())
         {
             mErrors.append(i18n("%1 is not a normal file.", m_fileAccess.prettyAbsPath()));
@@ -374,25 +360,24 @@ void SourceData::readAndPreprocess(QTextCodec* pEncoding, bool bAutoDetectUnicod
 
             fileNameIn1 = m_tempInputFileName;
         }
-        if(bAutoDetectUnicode)
+        if(bAutoDetect)
         {
-            m_pEncoding = detectEncoding(fileNameIn1, pEncoding);
+            mEncoding = detectEncoding(fileNameIn1).value_or(encoding);
         }
     }
     else // The input was set via setData(), probably from clipboard.
     {
         /*
-            Happens during initial startup. This means we have nothing to compare
+            Used to happen during early startup this is now a bug.
         */
-        if(m_tempInputFileName.isEmpty())
-            return;
+        assert(!m_tempInputFileName.isEmpty());
 
         fileNameIn1 = m_tempInputFileName;
-        m_pEncoding = QTextCodec::codecForName("UTF-8");
+        mEncoding = "UTF-8";
     }
-    QTextCodec* pEncoding1 = m_pEncoding;
-    QTextCodec* pEncoding2 = m_pEncoding;
-    const QString overSizedFile = i18nc("Read error message. %1 = filepath", "File %1 too large to process. Skipping.", fileNameIn1);
+    QByteArray pEncoding1 = getEncoding();
+    QByteArray pEncoding2 = getEncoding();
+    const QString overSizedFile = i18nc("Error message. %1 = filepath", "File %1 too large to process. Skipping.", fileNameIn1);
 
     m_normalData.reset();
     m_lmppData.reset();
@@ -405,7 +390,7 @@ void SourceData::readAndPreprocess(QTextCodec* pEncoding, bool bAutoDetectUnicod
         try
         {
             // Run the first preprocessor
-            if(m_pOptions->m_PreProcessorCmd.isEmpty())
+            if(gOptions->m_PreProcessorCmd.isEmpty())
             {
                 // No preprocessing: Read the file directly:
                 if(!m_normalData.readFile(faIn))
@@ -428,16 +413,16 @@ void SourceData::readAndPreprocess(QTextCodec* pEncoding, bool bAutoDetectUnicod
                 QTemporaryFile tmpInPPFile;
                 QString fileNameInPP = fileNameIn1;
 
-                if(pEncoding1 != m_pOptions->m_pEncodingPP)
+                if(pEncoding1 != gOptions->mEncodingPP)
                 {
                     // Before running the preprocessor convert to the format that the preprocessor expects.
                     FileAccess::createTempFile(tmpInPPFile);
                     fileNameInPP = tmpInPPFile.fileName();
-                    pEncoding1 = m_pOptions->m_pEncodingPP;
-                    convertFileEncoding(fileNameIn1, pEncoding, fileNameInPP, pEncoding1);
+                    pEncoding1 = gOptions->mEncodingPP;
+                    convertFileEncoding(fileNameIn1, encoding, fileNameInPP, pEncoding1);
                 }
 
-                QString ppCmd = m_pOptions->m_PreProcessorCmd;
+                QString ppCmd = gOptions->m_PreProcessorCmd;
                 FileAccess::createTempFile(fileOut1);
                 fileNameOut1 = fileOut1.fileName();
 
@@ -460,15 +445,16 @@ void SourceData::readAndPreprocess(QTextCodec* pEncoding, bool bAutoDetectUnicod
                 {
                     mErrors.append(
                         i18n("Preprocessing possibly failed. Check this command:\n\n  %1"
-                             "\n\nThe preprocessing command will be disabled now.", ppCmd) +
+                             "\n\nThe preprocessing command will be disabled now.",
+                             ppCmd) +
                         errorReason);
-                    m_pOptions->m_PreProcessorCmd = "";
+                    gOptions->m_PreProcessorCmd = "";
 
-                    pEncoding1 = m_pEncoding;
+                    pEncoding1 = getEncoding();
                 }
             }
         }
-        catch(std::bad_alloc&)
+        catch(const std::bad_alloc&)
         {
             m_normalData.reset();
             mErrors.append(overSizedFile);
@@ -485,22 +471,22 @@ void SourceData::readAndPreprocess(QTextCodec* pEncoding, bool bAutoDetectUnicod
             return;
 
         // LineMatching Preprocessor
-        if(!m_pOptions->m_LineMatchingPreProcessorCmd.isEmpty())
+        if(!gOptions->m_LineMatchingPreProcessorCmd.isEmpty())
         {
             QTemporaryFile tempOut2, fileInPP;
             fileNameIn2 = fileNameOut1.isEmpty() ? fileNameIn1 : fileNameOut1;
             QString fileNameInPP = fileNameIn2;
             pEncoding2 = pEncoding1;
-            if(pEncoding2 != m_pOptions->m_pEncodingPP)
+            if(pEncoding2 != gOptions->mEncodingPP)
             {
                 // Before running the preprocessor convert to the format that the preprocessor expects.
                 FileAccess::createTempFile(fileInPP);
                 fileNameInPP = fileInPP.fileName();
-                pEncoding2 = m_pOptions->m_pEncodingPP;
+                pEncoding2 = gOptions->mEncodingPP;
                 convertFileEncoding(fileNameIn2, pEncoding1, fileNameInPP, pEncoding2);
             }
 
-            QString ppCmd = m_pOptions->m_LineMatchingPreProcessorCmd;
+            QString ppCmd = gOptions->m_LineMatchingPreProcessorCmd;
             FileAccess::createTempFile(tempOut2);
             fileNameOut2 = tempOut2.fileName();
             QProcess ppProcess;
@@ -524,7 +510,7 @@ void SourceData::readAndPreprocess(QTextCodec* pEncoding, bool bAutoDetectUnicod
                     i18n("The line-matching-preprocessing possibly failed. Check this command:\n\n  %1"
                          "\n\nThe line-matching-preprocessing command will be disabled now.", ppCmd) +
                     errorReason);
-                m_pOptions->m_LineMatchingPreProcessorCmd = "";
+                gOptions->m_LineMatchingPreProcessorCmd = "";
                 if(!m_lmppData.readFile(fileNameIn2))
                 {
                     mErrors.append(i18nc("Read error message. %1 = filepath", "Failed to read file: %1", fileNameIn2));
@@ -532,7 +518,7 @@ void SourceData::readAndPreprocess(QTextCodec* pEncoding, bool bAutoDetectUnicod
                 }
             }
         }
-        else if(m_pOptions->ignoreComments() || m_pOptions->m_bIgnoreCase)
+        else if(gOptions->ignoreComments() || gOptions->m_bIgnoreCase)
         {
             // We need a copy of the normal data.
             m_lmppData.copyBufFrom(m_normalData);
@@ -564,12 +550,11 @@ void SourceData::readAndPreprocess(QTextCodec* pEncoding, bool bAutoDetectUnicod
     }
 
     // Ignore comments
-    if(m_pOptions->ignoreComments() && hasData())
+    if(gOptions->ignoreComments() && hasData())
     {
         qint64 vSize = std::min(m_normalData.lineCount(), m_lmppData.lineCount());
-        assert(vSize < TYPE_MAX(qint32));
-        //Perform explcit cast to insure well defined behavior comparing 32-bit to a 64-bit value
-        for(qint32 i = 0; (qint64)i < vSize; ++i)
+
+        for(qint64 i = 0; i < vSize; ++i)
         {
             //TODO: Phase this out. We should not be messing with these flags outside the parser.
             (*m_normalData.m_v)[i].setPureComment((*m_lmppData.m_v)[i].isPureComment());
@@ -579,17 +564,14 @@ void SourceData::readAndPreprocess(QTextCodec* pEncoding, bool bAutoDetectUnicod
 }
 
 /** Prepare the linedata vector for every input line.*/
-bool SourceData::FileData::preprocess(QTextCodec* pEncoding, bool removeComments)
+bool SourceData::FileData::preprocess(const QByteArray& encoding, bool removeComments)
 {
     if(m_pBuf == nullptr)
         return true;
 
-    if(pEncoding == nullptr)
-        return false;
-
     QString line;
-    QChar curChar, prevChar = 0;
-    LineCount lineCount = 0;
+    QChar curChar, prevChar = '\0';
+    LineType lines = 0;
     QtSizeType lastOffset = 0;
     FileOffset skipBytes = 0;
     QScopedPointer<CommentParser> parser(new DefaultCommentParser());
@@ -598,160 +580,184 @@ bool SourceData::FileData::preprocess(QTextCodec* pEncoding, bool removeComments
     QVector<e_LineEndStyle> vOrigDataLineEndStyle;
     m_eLineEndStyle = eLineEndStyleUndefined;
 
-    QTextCodec* pCodec = detectEncoding(m_pBuf.get(), mDataSize, skipBytes);
-    if(pCodec != pEncoding)
+    QByteArray pCodec = detectEncoding(m_pBuf.get(), mDataSize, skipBytes).value_or(encoding);
+    if(pCodec != encoding)
         skipBytes = 0;
 
-    if(mDataSize - skipBytes > TYPE_MAX(qint32))
-        return false;
-
-    const QByteArray ba = QByteArray::fromRawData(m_pBuf.get() + skipBytes, (QtSizeType)(mDataSize - skipBytes));
-    QTextStream ts(ba, QIODevice::ReadOnly); // Don't use text mode we need to see the actual line ending.
-    ts.setCodec(pEncoding);
-    ts.setAutoDetectUnicode(false);
-
-    m_bIncompleteConversion = false;
-    m_unicodeBuf->clear();
-
-    assert(m_unicodeBuf->length() == 0);
-
-    mHasEOLTermination = false;
-    bool skipNextRead = false;
-    while(!ts.atEnd())
+    if(mDataSize - skipBytes > limits<qint32>::max())
     {
-        line.clear();
-        if(lineCount >= TYPE_MAX(LineCount) - 5)
+        reset();
+        return false;
+    }
+
+    try
+    {
+        const QByteArray ba = QByteArray::fromRawData(m_pBuf.get() + skipBytes, (QtSizeType)(mDataSize - skipBytes));
+        EncodedDataStream ds(ba);
+
+        mHasBOM = skipBytes != 0;
+        ds.setEncoding(encoding);
+        ds.setGenerateByteOrderMark(skipBytes != 0);
+
+        m_bIncompleteConversion = false;
+        m_unicodeBuf->clear();
+
+        assert(m_unicodeBuf->length() == 0);
+
+        mHasEOLTermination = false;
+        bool skipNextRead = false;
+        while(!ds.atEnd())
         {
-            m_v->clear();
-            return false;
-        }
-
-        if(!skipNextRead){
-            prevChar = curChar;
-            curChar = ts.read(1).unicode()[0];
-        }
-        else
-            skipNextRead = false;
-
-        QtSizeType  firstNonwhite = 0;
-        bool        foundNonWhite = false;
-
-        //QTextStream::readLine doesn't tell us about line endings.
-        while(curChar != '\n' && curChar != '\r')
-        {
-            if(curChar.isNull() || curChar.isNonCharacter())
+            line.clear();
+            if(lines >= limits<LineType>::max() - 5)
             {
-                m_v->clear();
-                return true;
+                reset();
+                return false;
             }
 
-            if(curChar == QChar::ReplacementCharacter)
-                m_bIncompleteConversion = true;
-
-            line.append(curChar);
-            if(!curChar.isSpace() && !foundNonWhite)
+            if(!skipNextRead)
             {
-                firstNonwhite = line.length();
-                foundNonWhite = true;
+                prevChar = curChar;
+                ds.readChar(curChar);
             }
+            else
+                skipNextRead = false;
 
-            if(ts.atEnd())
-                break;
+            QtSizeType firstNonwhite = 0;
+            bool foundNonWhite = false;
 
-            prevChar = curChar;
-            curChar = ts.read(1).unicode()[0];
-        }
-
-        switch(curChar.unicode())
-        {
-            case '\n':
-                vOrigDataLineEndStyle.push_back(eLineEndStyleUnix);
-                break;
-            case '\r':
-                if((FileOffset)lastOffset < mDataSize)
+            while(curChar != '\n' && curChar != '\r')
+            {
+                if(curChar.isNull() || curChar.isNonCharacter())
                 {
-                    prevChar = curChar;
-                    curChar = ts.read(1).unicode()[0];
-
-                    if(curChar == '\n')
-                    {
-                        vOrigDataLineEndStyle.push_back(eLineEndStyleDos);
-                        break;
-                    }
-                    //work around for lack of seek API in QTextStream
-                    skipNextRead = true;
+                    m_v->clear();
+                    return true;
                 }
 
-                //old mac style ending.
-                vOrigDataLineEndStyle.push_back(eLineEndStyleUndefined);
-                break;
-        }
-        parser->processLine(line);
-        if(removeComments)
-            parser->removeComment(line);
+                if(curChar == QChar::ReplacementCharacter)
+                    m_bIncompleteConversion = true;
 
-        ++lineCount;
-        m_v->push_back(LineData(m_unicodeBuf, lastOffset, line.length(), firstNonwhite, parser->isSkipable(), parser->isPureComment()));
-        //The last line may not have an EOL mark. In that case don't add one to our buffer.
-        m_unicodeBuf->append(line);
-        if(curChar == '\n' || curChar == '\r' || prevChar == '\r')
+                line.append(curChar);
+                if(!curChar.isSpace() && !foundNonWhite)
+                {
+                    firstNonwhite = line.length();
+                    foundNonWhite = true;
+                }
+
+                if(ds.atEnd())
+                    break;
+
+                prevChar = curChar;
+                ds.readChar(curChar);
+            }
+
+            switch(curChar.unicode())
+            {
+                case '\n':
+                    vOrigDataLineEndStyle.push_back(eLineEndStyleUnix);
+                    break;
+                case '\r':
+                    if((FileOffset)lastOffset < mDataSize)
+                    {
+                        prevChar = curChar;
+                        ds.readChar(curChar);
+
+                        if(curChar == '\n')
+                        {
+                            vOrigDataLineEndStyle.push_back(eLineEndStyleDos);
+                            break;
+                        }
+                        //work around for lack of seek API in QDataStream
+                        skipNextRead = true;
+                    }
+
+                    //old mac style ending.
+                    vOrigDataLineEndStyle.push_back(eLineEndStyleOldMac);
+                    break;
+            }
+            parser->processLine(line);
+            if(removeComments)
+                parser->removeComment(line);
+            //Qt6 intrudes 64bit sizes
+            if(line.size() >= limits<LineType>::max())
+            {
+                reset();
+                return false;
+            }
+
+            ++lines;
+            m_v->push_back(LineData(m_unicodeBuf, lastOffset, line.length(), firstNonwhite, parser->isSkipable(), parser->isPureComment()));
+            //The last line may not have an EOL mark. In that case don't add one to our buffer.
+            m_unicodeBuf->append(line);
+            if(curChar == '\n' || curChar == '\r' || prevChar == '\r')
+            {
+                //kdiff3 internally uses only unix style endings for simplicity.
+                m_unicodeBuf->append('\n');
+            }
+
+            assert(m_unicodeBuf->length() != lastOffset);
+            lastOffset = m_unicodeBuf->length();
+        }
+
+        /*
+            Process trailing new line as if there were a blank non-terminated line after it.
+            But do nothing to the data buffer since this is a phantom line needed for internal purposes.
+        */
+        if(curChar == '\n' || curChar == '\r')
         {
-            //kdiff3 internally uses only unix style endings for simplicity.
-            m_unicodeBuf->append('\n');
+            mHasEOLTermination = true;
+            ++lines;
+
+            parser->processLine("");
+            m_v->push_back(LineData(m_unicodeBuf, lastOffset, 0, 0, parser->isSkipable(), parser->isPureComment()));
         }
 
-        assert(m_unicodeBuf->length() != lastOffset);
-        lastOffset = m_unicodeBuf->length();
-    }
+        m_v->push_back(LineData(m_unicodeBuf, lastOffset));
 
-    /*
-        Process trailing new line as if there were a blank non-terminated line after it.
-        But do nothing to the data buffer since this is a phantom line needed for internal purposes.
-    */
-    if(curChar == '\n' || curChar == '\r')
+        m_bIsText = true;
+
+        if(!vOrigDataLineEndStyle.isEmpty())
+            m_eLineEndStyle = vOrigDataLineEndStyle[0];
+
+        mLineCount = lines;
+        return true;
+    }
+    catch(const std::bad_alloc&)
     {
-        mHasEOLTermination = true;
-        ++lineCount;
-
-        parser->processLine("");
-        m_v->push_back(LineData(m_unicodeBuf, lastOffset, 0, 0, parser->isSkipable(), parser->isPureComment()));
+        reset();
+        return false;
     }
-
-    m_v->push_back(LineData(m_unicodeBuf, lastOffset));
-
-    m_bIsText = true;
-
-    if(!vOrigDataLineEndStyle.isEmpty())
-        m_eLineEndStyle = vOrigDataLineEndStyle[0];
-
-    mLineCount = lineCount;
-    return true;
 }
 
 // Convert the input file from input encoding to output encoding and write it to the output file.
-bool SourceData::convertFileEncoding(const QString& fileNameIn, QTextCodec* pCodecIn,
-                                     const QString& fileNameOut, QTextCodec* pCodecOut)
+bool SourceData::convertFileEncoding(const QString& fileNameIn, const QByteArray& pCodecIn,
+                                     const QString& fileNameOut, const QByteArray& pCodecOut)
 {
     QFile in(fileNameIn);
     if(!in.open(QIODevice::ReadOnly))
         return false;
-    QTextStream inStream(&in);
-    inStream.setCodec(pCodecIn);
-    inStream.setAutoDetectUnicode(false);
+    EncodedDataStream inStream(&in);
+    inStream.setEncoding(pCodecIn);
 
     QFile out(fileNameOut);
     if(!out.open(QIODevice::WriteOnly))
         return false;
-    QTextStream outStream(&out);
-    outStream.setCodec(pCodecOut);
+    EncodedDataStream outStream(&out);
+    outStream.setEncoding(pCodecOut);
 
-    QString data = inStream.readAll();
+    QString data;
+    while(!inStream.atEnd())
+    {
+        QChar c;
+        inStream.readChar(c);
+        data += c;
+    }
     outStream << data;
 
     return true;
 }
 
-QTextCodec* SourceData::getEncodingFromTag(const QByteArray& s, const QByteArray& encodingTag)
+std::optional<const QByteArray> SourceData::getEncodingFromTag(const QByteArray& s, const QByteArray& encodingTag)
 {
     QtSizeType encodingPos = s.indexOf(encodingTag);
     if(encodingPos >= 0)
@@ -769,31 +775,33 @@ QTextCodec* SourceData::getEncodingFromTag(const QByteArray& s, const QByteArray
         if(encodingEnd >= 0) // e.g.: <meta charset="utf-8"> or <?xml version="1.0" encoding="ISO-8859-1"?>
         {
             QByteArray encoding = s.mid(apostrophPos + 1, encodingEnd - (apostrophPos + 1));
-            return QTextCodec::codecForName(encoding);
+            if(QTextCodec::codecForName(encoding))
+                return encoding;
         }
         else // e.g.: <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
         {
             QByteArray encoding = s.mid(encodingPos + encodingTag.length(), apostrophPos - (encodingPos + encodingTag.length()));
-            return QTextCodec::codecForName(encoding);
+            if(QTextCodec::codecForName(encoding))
+                return encoding;
         }
     }
-    return nullptr;
+    return {};
 }
 
-QTextCodec* SourceData::detectEncoding(const char* buf, qint64 size, FileOffset& skipBytes)
+std::optional<const QByteArray> SourceData::detectEncoding(const char* buf, qint64 size, FileOffset& skipBytes)
 {
     if(size >= 2)
     {
         if(buf[0] == '\xFF' && buf[1] == '\xFE')
         {
             skipBytes = 2;
-            return QTextCodec::codecForName("UTF-16LE");
+            return "UTF-16LE";
         }
 
         if(buf[0] == '\xFE' && buf[1] == '\xFF')
         {
             skipBytes = 2;
-            return QTextCodec::codecForName("UTF-16BE");
+            return "UTF-16BE";
         }
     }
     if(size >= 3)
@@ -801,8 +809,7 @@ QTextCodec* SourceData::detectEncoding(const char* buf, qint64 size, FileOffset&
         if(buf[0] == '\xEF' && buf[1] == '\xBB' && buf[2] == '\xBF')
         {
             skipBytes = 3;
-            //Custom codec.
-            return QTextCodec::codecForName("UTF-8-BOM");
+            return "UTF-8-BOM";
         }
     }
     skipBytes = 0;
@@ -821,9 +828,9 @@ QTextCodec* SourceData::detectEncoding(const char* buf, qint64 size, FileOffset&
         QtSizeType xmlHeaderEnd = s.indexOf("?>", xmlHeaderPos);
         if(xmlHeaderEnd >= 0)
         {
-            QTextCodec* pCodec = getEncodingFromTag(s.mid(xmlHeaderPos, xmlHeaderEnd - xmlHeaderPos), "encoding=");
-            if(pCodec)
-                return pCodec;
+            std::optional<const char*> encoding = getEncodingFromTag(s.mid(xmlHeaderPos, xmlHeaderEnd - xmlHeaderPos), "encoding=");
+            if(encoding.has_value())
+                return encoding;
         }
     }
     else // HTML
@@ -834,9 +841,9 @@ QTextCodec* SourceData::detectEncoding(const char* buf, qint64 size, FileOffset&
             QtSizeType metaHeaderEnd = s.indexOf(">", metaHeaderPos);
             if(metaHeaderEnd >= 0)
             {
-                QTextCodec* pCodec = getEncodingFromTag(s.mid(metaHeaderPos, metaHeaderEnd - metaHeaderPos), "charset=");
-                if(pCodec)
-                    return pCodec;
+                std::optional<const char*> encoding = getEncodingFromTag(s.mid(metaHeaderPos, metaHeaderEnd - metaHeaderPos), "charset=");
+                if(encoding.has_value())
+                    return encoding;
 
                 metaHeaderPos = s.indexOf("<meta", metaHeaderEnd);
             }
@@ -845,20 +852,20 @@ QTextCodec* SourceData::detectEncoding(const char* buf, qint64 size, FileOffset&
         }
     }
     //Attempt to detect non-bom UTF8. This is a very common encoding.
-    return dectectUTF8(s);
+    return detectUTF8(s);
 }
 
-QTextCodec* SourceData::dectectUTF8(const QByteArray& data)
+std::optional<const QByteArray> SourceData::detectUTF8(const QByteArray& data)
 {
     QTextCodec* utf8 = QTextCodec::codecForName("UTF-8");
 
     QTextCodec::ConverterState state;
-    utf8->toUnicode(data.constData(), data.size(), &state);
+    utf8->toUnicode(data.constData(), SafeInt<qint32>(data.size()), &state);
 
     if(state.invalidChars == 0)
         for (qint32 i = 0; i < data.size()-state.remainingChars; i++)
             if ((unsigned)data.at(i) > 127)
-                return utf8;
+                return "UTF-8";
 
-    return nullptr;
+    return {};
 }
