@@ -574,15 +574,20 @@ LineRef DiffTextWindow::convertDiff3LineIdxToLine(const LineType d3lIdx) const
 */
 LineRef getBestFirstLine(LineRef line, LineType nofLines, LineRef firstLine, LineType visibleLines)
 {
-    LineRef newFirstLine = firstLine;
-    if(line < firstLine || line + nofLines + 2 > firstLine + visibleLines)
-    {
-        if(nofLines > visibleLines || nofLines <= (2 * visibleLines / 3 - 1))
-            newFirstLine = line - visibleLines / 3;
-        else
-            newFirstLine = line - (visibleLines - nofLines);
-    }
+    if(line < visibleLines) //well known result.
+        return 0;
 
+    LineRef newFirstLine = firstLine;
+    if(line > firstLine && line + nofLines + 2 <= firstLine + visibleLines)
+        return newFirstLine;
+
+    if(nofLines < visibleLines)
+        newFirstLine = std::max(0, (LineType)std::ceil(line - (visibleLines - nofLines) / 2));
+    else
+    {
+        qint32 numberPages = floor(nofLines / visibleLines);
+        newFirstLine = std::max(0, line - (visibleLines * numberPages) / 3);
+    }
     return newFirstLine;
 }
 
@@ -1393,7 +1398,7 @@ void DiffTextWindow::resizeEvent(QResizeEvent* e)
 {
     QSize newSize = e->size();
     QFontMetrics fm = fontMetrics();
-    qint32 visibleLines = newSize.height() / fm.lineSpacing() - 2;
+    LineType visibleLines = newSize.height() / fm.lineSpacing() - 2;
     //TODO: Fix after line number area is converted to a QWidget.
     qint32 visibleColumns = newSize.width() / fm.horizontalAdvance('0') - d->leftInfoWidth();
 
