@@ -10,8 +10,9 @@
 #include "fileaccess.h"
 
 #include "common.h"
+#include "compat.h"
 
-#ifndef AUTOTEST
+#if HAS_KFKIO && !defined AUTOTEST
 #include "DefaultFileAccessJobHandler.h"
 #endif
 #include "FileAccessJobHandler.h"
@@ -34,10 +35,7 @@
 #include <QTemporaryFile>
 #include <QtMath>
 
-#include <KLocalizedString>
-
-//This triggers template instantiation even if set to default.
-#ifndef AUTOTEST
+#if HAS_KFKIO && !defined AUTOTEST
 FileAccess::FileAccess()
 {
     mJobHandler.reset(new DefaultFileAccessJobHandler(this));
@@ -260,7 +258,7 @@ void FileAccess::reset()
 void FileAccess::setFile(FileAccess* pParent, const QFileInfo& fi)
 {
     assert(pParent != this);
-#ifndef AUTOTEST
+#if HAS_KFKIO && !defined AUTOTEST
     if(mJobHandler == nullptr) mJobHandler.reset(new DefaultFileAccessJobHandler(this));
 #endif
     reset();
@@ -277,7 +275,7 @@ void FileAccess::setFile(const QString& name, bool bWantToWrite)
     if(name.isEmpty())
         return;
 
-    QUrl url = QUrl::fromUserInput(name, QString(), QUrl::AssumeLocalFile);
+    QUrl url = QUrl::fromUserInput(name, QDir::currentPath(), QUrl::AssumeLocalFile);
     setFile(url, bWantToWrite);
 }
 
@@ -286,7 +284,7 @@ void FileAccess::setFile(const QUrl& url, bool bWantToWrite)
     if(url.isEmpty())
         return;
 
-#ifndef AUTOTEST
+#if HAS_KFKIO && !defined AUTOTEST
     if(mJobHandler == nullptr) mJobHandler.reset(new DefaultFileAccessJobHandler(this));
 #endif
     reset();
@@ -352,19 +350,18 @@ void FileAccess::loadData()
 
 #ifndef Q_OS_WIN
         // Unfortunately Qt5 symLinkTarget/readLink always return an absolute path, even if the link is relative
-        char* s = (char*)malloc(PATH_MAX + 1);
-        ssize_t len = readlink(QFile::encodeName(absoluteFilePath()).constData(), s, PATH_MAX);
+        std::unique_ptr<char[]> s = std::make_unique<char[]>(PATH_MAX + 1);
+        ssize_t len = readlink(QFile::encodeName(absoluteFilePath()).constData(), s.get(), PATH_MAX);
         if(len > 0)
         {
             s[len] = '\0';
-            m_linkTarget = QFile::decodeName(s);
+            m_linkTarget = QFile::decodeName(s.get());
         }
-        free(s);
 #endif
 
         m_bBrokenLink = !QFileInfo::exists(m_linkTarget);
         //We want to know if the link itself exists
-        if(!m_bExists)
+        if(m_bBrokenLink)
             m_bExists = true;
 
         if(!m_modificationTime.isValid())
@@ -393,7 +390,7 @@ void FileAccess::addPath(const QString& txt, bool reinit)
     }
 }
 
-#ifndef AUTOTEST
+#if HAS_KFKIO && !defined AUTOTEST
 /*     Filetype:
        S_IFMT     0170000   bitmask for the file type bitfields
        S_IFSOCK   0140000   socket
@@ -521,11 +518,11 @@ void FileAccess::setFromUdsEntry(const KIO::UDSEntry& e, FileAccess* parent)
         */
         m_url = parent->url();
         addPath(filePath, false);
-        //Not something I expect to happen but can't rule it out either
+        //This too would be a bug somewhere. Don't crash out though.
         if(Q_UNLIKELY(m_url == parent->url()))
         {
             m_url.clear();
-            qCritical() << "Parent and child could not be distinguished.";
+            qCCritical(kdiffFileAccess) << "Parent and child could not be distinguished.";
             return;
         }
 
@@ -652,7 +649,7 @@ qint64 FileAccess::size() const
         return m_fileInfo.size();
 }
 
-QUrl FileAccess::url() const
+const QUrl& FileAccess::url() const
 {
     return m_url;
 }
@@ -701,7 +698,7 @@ bool FileAccess::isHidden() const
         return m_fileInfo.isHidden();
 }
 
-QString FileAccess::readLink() const
+const QString& FileAccess::readLink() const
 {
     return m_linkTarget;
 }
@@ -715,7 +712,7 @@ QString FileAccess::absoluteFilePath() const
 } // Full abs path
 
 // Just the name-part of the path, without parent directories
-QString FileAccess::fileName(bool needTmp) const
+const QString& FileAccess::fileName(bool needTmp) const
 {
     if(!isLocal())
         return (needTmp) ? m_localCopy : m_name;
@@ -743,7 +740,7 @@ QString FileAccess::fileRelPath() const
 
         const FileAccess* curEntry = this;
         path = fileName();
-        //Avoid recursing to FileAccess::fileRelPath or we can get very large stacks.
+        //Avoid recursively calling FileAccess::fileRelPath or we can get very large stacks.
         curEntry = curEntry->parent();
         while(curEntry != nullptr)
         {
@@ -767,7 +764,7 @@ QString FileAccess::prettyAbsPath() const
     return isLocal() ? absoluteFilePath() : m_url.toDisplayString();
 }
 
-QDateTime FileAccess::lastModified() const
+const QDateTime& FileAccess::lastModified() const
 {
     return m_modificationTime;
 }
@@ -898,7 +895,7 @@ bool FileAccess::removeFile()
 
 bool FileAccess::listDir(DirectoryList* pDirList, bool bRecursive, bool bFindHidden,
                          const QString& filePattern, const QString& fileAntiPattern, const QString& dirAntiPattern,
-                         bool bFollowDirLinks, IgnoreList& ignoreList)
+                         bool bFollowDirLinks, IgnoreList& ignoreList) const
 {
     assert(mJobHandler != nullptr);
     return mJobHandler->listDir(pDirList, bRecursive, bFindHidden, filePattern, fileAntiPattern,
@@ -1002,7 +999,7 @@ void FileAccess::createTempFile(QTemporaryFile& tmpFile)
     tmpFile.close();
 }
 
-#ifndef AUTOTEST
+#if HAS_KFKIO && !defined AUTOTEST
 bool FileAccess::makeDir(const QString& dirName)
 {
     return DefaultFileAccessJobHandler::mkDir(dirName);
@@ -1025,7 +1022,7 @@ bool FileAccess::symLink(const QString& linkTarget, const QString& linkLocation)
 
 bool FileAccess::exists(const QString& name)
 {
-    FileAccess fa(name);
+    const FileAccess fa(name);
     return fa.exists();
 }
 
@@ -1037,8 +1034,8 @@ qint64 FileAccess::sizeForReading()
         // Size couldn't be determined. Copy the file to a local temp place.
         if(createLocalCopy())
         {
-            QString localCopy = tmpFile->fileName();
-            QFileInfo fi(localCopy);
+            const QString localCopy = tmpFile->fileName();
+            const QFileInfo fi(localCopy);
 
             m_size = fi.size();
             m_localCopy = localCopy;
@@ -1069,7 +1066,7 @@ QString FileAccess::cleanPath(const QString& path) // static
         Tell Qt to treat the supplied path as user input otherwise it will not make useful decisions
         about how to convert from the possibly local or remote "path" string to QUrl.
     */
-    QUrl url = QUrl::fromUserInput(path, QString(), QUrl::AssumeLocalFile);
+    const QUrl url = QUrl::fromUserInput(path, QString(), QUrl::AssumeLocalFile);
 
     if(FileAccess::isLocal(url))
     {
@@ -1086,7 +1083,7 @@ bool FileAccess::createBackup(const QString& bakExtension)
     if(exists())
     {
         // First rename the existing file to the bak-file. If a bak-file file exists, delete that.
-        QString bakName = absoluteFilePath() + bakExtension;
+        const QString bakName = absoluteFilePath() + bakExtension;
         FileAccess bakFile(bakName, true /*bWantToWrite*/);
         if(bakFile.exists())
         {
@@ -1116,7 +1113,7 @@ void FileAccess::doError()
 
 void FileAccess::filterList(const QString& dir, DirectoryList* pDirList, const QString& filePattern,
                             const QString& fileAntiPattern, const QString& dirAntiPattern,
-                            IgnoreList& ignoreList)
+                            const IgnoreList& ignoreList)
 {
     //TODO: Ask os for this information don't hard code it.
 #if defined(Q_OS_WIN)
@@ -1126,12 +1123,12 @@ void FileAccess::filterList(const QString& dir, DirectoryList* pDirList, const Q
 #endif
 
     // Now remove all entries that should be ignored:
-    DirectoryList::iterator i;
-    for(i = pDirList->begin(); i != pDirList->end();)
+    DirectoryList::const_iterator i;
+    for(i = pDirList->cbegin(); i != pDirList->cend();)
     {
-        DirectoryList::iterator i2 = i;
+        DirectoryList::const_iterator i2 = i;
         ++i2;
-        QString fileName = i->fileName();
+        const QString& fileName = i->fileName();
 
         if((i->isFile() &&
             (!Utils::wildcardMultiMatch(filePattern, fileName, bCaseSensitive) ||
